%define anolis_release 2

# disable tests in mock, tests can pass on local rpmbuild
%bcond_with    tests

%global redis_modules_abi 1
%global redis_modules_dir %{_libdir}/%{name}/modules
%global doc_commit ab025f62797f2622ddad7bad938e747bebb6ab09
%global short_doc_commit %(c=%{doc_commit}; echo ${c:0:7})
%global macrosdir %{_rpmconfigdir}/macros.d
%global make_flags DEBUG="" V="echo" LDFLAGS="%{?__global_ldflags}" CFLAGS+="%{optflags} -fPIC" INSTALL="install -p" PREFIX=%{buildroot}%{_prefix} BUILD_WITH_SYSTEMD=yes BUILD_TLS=yes

Name:              redis
Version:           7.0.13
Release:           %{anolis_release}%{?dist}
Summary:           A persistent key-value database
License:           BSD and MIT
URL:               https://redis.io
Source0:           https://download.redis.io/releases/%{name}-%{version}.tar.gz
Source1:           redis.logrotate
Source2:           redis-sentinel.service
Source3:           redis.service
Source7:           redis-limit-systemd
Source9:           macros.redis
Source10:          https://github.com/%{name}/%{name}-doc/archive/%{doc_commit}/%{name}-doc-%{short_doc_commit}.tar.gz

Patch0001:         0001-1st-man-pageis-for-redis-cli-redis-benchmark-redis-c.patch
Patch0002:         0001-add-loongarch-support-for-jemalloc.patch

BuildRequires: make gcc systemd-devel openssl-devel systemd-rpm-macros
%if %{with tests}
BuildRequires:     procps-ng tcl
%endif

Requires:          logrotate
Requires(pre):     shadow-utils
Requires(post):    systemd
Requires(preun):   systemd
Requires(postun):  systemd

Obsoletes:         redis-trib < 5
Provides:          bundled(lzf)
Provides:          bundled(hiredis) = 0.14.0
Provides:          bundled(jemalloc) = 5.2.1
Provides:          bundled(lua-libs) = 5.1.5
Provides:          bundled(linenoise) = 1.0
Provides:          bundled(hdr_histogram) = 0.11.0
Provides:          redis(modules_abi) = %{redis_modules_abi}

%description
Redis is often referred to as a data structures server. What this means is that Redis 
provides access to mutable data structures via a set of commands, which are sent using 
a server-client model with TCP sockets and a simple protocol. So different processes 
can query and modify the same data structures in a shared way.

Data structures implemented into Redis have a few special properties:

  - Redis cares to store them on disk, even if they are always served and modified into the 
    server memory. This means that Redis is fast, but that it is also non-volatile.
  - The implementation of data structures emphasizes memory efficiency, so data structures 
    inside Redis will likely use less memory compared to the same data structure modelled 
    using a high-level programming language.
  - Redis offers a number of features that are natural to find in a database, like replication, 
    tunable levels of durability, clustering, and high availability.

Another good example is to think of Redis as a more complex version of memcached, where the operations 
are not just SETs and GETs, but operations that work with complex data types like Lists, Sets, ordered 
data structures, and so forth.

%package           devel
Summary:           Development header for Redis module development
Provides:          %{name}-static = %{version}-%{release}

%description       devel
Header file required for building loadable Redis modules. Detailed API 
documentation is available in the redis-doc package.

%package            doc
Summary:            Documentation for Redis including man pages
License:            CC-BY-SA
BuildArch:          noarch
Requires:           %{name} = %{EVR}

Conflicts:         redis < 4.0

%description doc
Manual pages and detailed documentation for many aspects of Redis use,
administration and development.

%prep
%setup -q -b 10
%setup -q
mv ../%{name}-doc-%{doc_commit} doc
%patch -P0001 -p1
%patch -P0002 -p1

mv deps/lua/COPYRIGHT    COPYRIGHT-lua
mv deps/jemalloc/COPYING COPYING-jemalloc
mv deps/hiredis/COPYING  COPYING-hiredis
mv deps/hdr_histogram/LICENSE.txt LICENSE-hdrhistogram
mv deps/hdr_histogram/COPYING.txt COPYING-hdrhistogram

# Configuration file changes
sed -i -e 's|^logfile .*$|logfile /var/log/redis/redis.log|g' redis.conf
sed -i -e 's|^dir .*$|dir /var/lib/redis|g' redis.conf
sed -i -e 's|^logfile .*$|logfile /var/log/redis/sentinel.log|g' sentinel.conf

# Module API version safety check
api=`sed -n -e 's/#define REDISMODULE_APIVER_[0-9][0-9]* //p' src/redismodule.h`
if test "$api" != "%{redis_modules_abi}"; then
   : Error: Upstream API version is now ${api}, expecting %%{redis_modules_abi}.
   : Update the redis_modules_abi macro, the rpmmacros file, and rebuild.
   exit 1
fi

%build
%make_build %{make_flags} all

%install
make %{make_flags} install

# Filesystem.
mkdir -p %{buildroot}%{_sharedstatedir}/%{name}
mkdir -p %{buildroot}%{_localstatedir}/log/%{name}
mkdir -p %{buildroot}%{_localstatedir}/run/%{name}
mkdir -p %{buildroot}%{redis_modules_dir}

install -pDm644 %{SOURCE1} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}
install -pDm640 %{name}.conf  %{buildroot}%{_sysconfdir}/%{name}/%{name}.conf
install -pDm640 sentinel.conf %{buildroot}%{_sysconfdir}/%{name}/sentinel.conf
mkdir -p %{buildroot}%{_unitdir}
install -pm644 %{SOURCE3} %{buildroot}%{_unitdir}
install -pm644 %{SOURCE2} %{buildroot}%{_unitdir}
install -p -D -m 644 %{SOURCE7} %{buildroot}%{_sysconfdir}/systemd/system/%{name}.service.d/limit.conf
install -p -D -m 644 %{SOURCE7} %{buildroot}%{_sysconfdir}/systemd/system/%{name}-sentinel.service.d/limit.conf
chmod 755 %{buildroot}%{_bindir}/%{name}-*
install -pDm644 src/%{name}module.h %{buildroot}%{_includedir}/%{name}module.h

# Install rpm macros for redis modules
mkdir -p %{buildroot}%{macrosdir}
install -pDm644 %{SOURCE9} %{buildroot}%{macrosdir}/macros.%{name}

# Install man pages
man=$(dirname %{buildroot}%{_mandir})
for page in man/man?/*; do
    install -Dpm644 $page $man/$page
done
ln -s redis-server.1 %{buildroot}%{_mandir}/man1/redis-sentinel.1
ln -s redis.conf.5   %{buildroot}%{_mandir}/man5/redis-sentinel.conf.5

# Install documentation and html pages
doc=$(echo %{buildroot}/%{_docdir}/%{name})
for page in 00-RELEASENOTES BUGS MANIFESTO *.md; do
    install -Dpm644 $page $doc/$page
done
for page in $(find doc -name \*.md | sed -e 's|.md$||g'); do
    base=$(echo $page | sed -e 's|doc/||g')
    install -Dpm644 $page.md $doc/$base.md
done

%check
%if %{with tests}
taskset -c 1 make %{make_flags} test
make %{make_flags} test-sentinel
%endif

%generate_compatibility_deps

%pre
getent group %{name} &> /dev/null || \
groupadd -r %{name} &> /dev/null
getent passwd %{name} &> /dev/null || \
useradd -r -g %{name} -d %{_sharedstatedir}/%{name} -s /sbin/nologin \
-c 'Redis Database Server' %{name} &> /dev/null
exit 0

%post
# migration for config files
if [ -f %{_sysconfdir}/%{name}.conf -a ! -L %{_sysconfdir}/%{name}.conf ]; then
  if [ -f %{_sysconfdir}/%{name}/%{name}.conf.rpmnew ]; then
    rm    %{_sysconfdir}/%{name}/%{name}.conf.rpmnew
  fi
  if [ -f %{_sysconfdir}/%{name}/%{name}.conf ]; then
    mv    %{_sysconfdir}/%{name}/%{name}.conf %{_sysconfdir}/%{name}/%{name}.conf.rpmnew
  fi
  mv %{_sysconfdir}/%{name}.conf %{_sysconfdir}/%{name}/%{name}.conf
  echo -e "\nWarning: %{name} configuration is now in %{_sysconfdir}/%{name} directory\n"
fi
if [ -f %{_sysconfdir}/%{name}-sentinel.conf  -a ! -L %{_sysconfdir}/%{name}-sentinel.conf  ]; then
  if [ -f %{_sysconfdir}/%{name}/sentinel.conf.rpmnew ]; then
    rm    %{_sysconfdir}/%{name}/sentinel.conf.rpmnew
  fi
  if [ -f %{_sysconfdir}/%{name}/sentinel.conf ]; then
    mv    %{_sysconfdir}/%{name}/sentinel.conf %{_sysconfdir}/%{name}/sentinel.conf.rpmnew
  fi
  mv %{_sysconfdir}/%{name}-sentinel.conf %{_sysconfdir}/%{name}/sentinel.conf
fi
%systemd_post %{name}.service
%systemd_post %{name}-sentinel.service

%preun
%systemd_preun %{name}.service
%systemd_preun %{name}-sentinel.service

%postun
%systemd_postun_with_restart %{name}.service
%systemd_postun_with_restart %{name}-sentinel.service

%files
%license COPYING COPYRIGHT-lua COPYING-jemalloc doc/LICENSE
%license COPYING-hiredis LICENSE-hdrhistogram COPYING-hdrhistogram
%config(noreplace) %{_sysconfdir}/logrotate.d/%{name}
%attr(0750, redis, root) %dir %{_sysconfdir}/%{name}
%attr(0640, redis, root) %config(noreplace) %{_sysconfdir}/%{name}/%{name}.conf
%attr(0640, redis, root) %config(noreplace) %{_sysconfdir}/%{name}/sentinel.conf
%dir %{_libdir}/%{name}
%dir %{redis_modules_dir}
%dir %{abidir}
%dir %attr(0750, redis, redis) %{_sharedstatedir}/%{name}
%dir %attr(0750, redis, redis) %{_localstatedir}/log/%{name}
%dir %{_sysconfdir}/systemd/system/%{name}.service.d
%dir %{_sysconfdir}/systemd/system/%{name}-sentinel.service.d
%dir %attr(0755, redis, redis) %ghost %{_localstatedir}/run/%{name}
%config(noreplace) %{_sysconfdir}/systemd/system/%{name}.service.d/limit.conf
%config(noreplace) %{_sysconfdir}/systemd/system/%{name}-sentinel.service.d/limit.conf
%exclude %{_docdir}/%{name}
%{_bindir}/%{name}-*
%{abidir}/%{name}*-option.list
%{_unitdir}/%{name}.service
%{_unitdir}/%{name}-sentinel.service
%{_mandir}/man1/%{name}*
%{_mandir}/man5/%{name}*

%files devel
%license COPYING
%{_includedir}/%{name}module.h
%{macrosdir}/*

%files doc
%docdir %{_docdir}/%{name}
%{_docdir}/%{name}

%changelog
* Sat Oct 7 2023 Jingyun Hua <huajingyun@loongson.cn> - 7.0.13-2
- Support loongarch64 for jemalloc

* Thu Sep 07 2023 Funda Wang <fundawang@yeah.net> - 7.0.13-1
- New version 7.0.13

* Tue Jul 11 2023 Funda Wang <fundawang@yeah.net> - 7.0.12-1
- New version 7.0.12

* Thu Apr 20 2023 Funda Wang <fundawang@yeah.net> - 7.0.11-1
- New version 7.0.11

* Wed Apr 12 2023 yuanhui <yuanhui@linux.alibaba.com> 7.0.10-2
- Optimize the spec file

* Tue Apr 04 2023 Funda Wang <fundawang@yeah.net> - 7.0.10-1
- New version 7.0.10
- drop redis-shutdown helper and rely on systemd

* Tue Jan 17 2023 Funda Wang <fundawang@yeah.net> - 7.0.8-1
- New version 7.0.8

* Fri Dec 16 2022 Funda Wang <fundawang@yeah.net> - 7.0.7-1
- New version 7.0.7

* Thu Nov 24 2022 Funda Wang <fundawang@yeah.net> - 7.0.5-1
- New version 7.0.5

* Wed Aug 31 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 7.0.4-1
- update redis to 7.0.4

* Fri May 13 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 7.0.0-1
- init from upstream
